@@ -16,19 +16,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
+"""Resource views
 """
-group views.
-
-$Id$
-"""
+from collections import defaultdict
 
 import z3c.form
 import z3c.form.browser.text
-from z3c.form import form, field, button, subform, validator, widget
-from z3c.form.interfaces import DISPLAY_MODE, HIDDEN_MODE, NO_VALUE, IActionHandler
+from z3c.form import form, field, button, widget
+from z3c.form.interfaces import DISPLAY_MODE, HIDDEN_MODE, NO_VALUE
 from zc.table import table
+from zc.table.column import GetterColumn
 
 from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
+from zope.cachedescriptors.property import Lazy
 from zope.component import adapter, adapts
 from zope.component import getUtilitiesFor
 from zope.component import queryAdapter
@@ -38,28 +38,43 @@ from zope.component import getMultiAdapter
 from zope.container.interfaces import INameChooser
 from zope.event import notify
 from zope.formlib import form as oldform
+from zope.i18n import translate
 from zope.interface import implementer, implements, implementsOnly
 from zope.lifecycleevent import ObjectCreatedEvent
 from zope.publisher.browser import BrowserView
 from zope.publisher.interfaces.browser import IBrowserRequest
+from zope.security.checker import canAccess
 from zope.session.interfaces import ISession
 from zope.traversing.browser.absoluteurl import absoluteURL
 from zope.traversing.browser.interfaces import IAbsoluteURL
 
-from schooltool.app.browser.app import BaseEditView
 from schooltool.app.interfaces import ISchoolToolApplication
-from schooltool.basicperson.browser.demographics import DemographicsView
+from schooltool.basicperson.browser.demographics import (
+    DemographicsView,
+    FlourishDemographicsView, FlourishReorderDemographicsView)
+from schooltool.basicperson.interfaces import IAddEditViewTitle
+from schooltool.basicperson.interfaces import ILimitKeysLabel
+from schooltool.basicperson.interfaces import ILimitKeysHint
+from schooltool.common.inlinept import InheritTemplate
 from schooltool.resource.interfaces import IBookingCalendar
-from schooltool.resource.interfaces import (IBaseResourceContained,
+from schooltool.resource.interfaces import (
              IResourceContainer, IResourceTypeInformation, IResourceSubTypes,
              IResource, IEquipment, ILocation, IResourceDemographicsFields)
 from schooltool.resource.resource import Resource, Location, Equipment
 from schooltool.table.interfaces import IFilterWidget
+from schooltool.table.interfaces import ITableFormatter
 from schooltool.table.table import url_cell_formatter
 from schooltool.table.table import CheckboxColumn
 from schooltool.table.table import FilterWidget
+from schooltool.table.table import SchoolToolTableFormatter
 from schooltool.person.browser.person import PersonFilterWidget
 from schooltool.resource.interfaces import IResourceFactoryUtility
+from schooltool.schoolyear.interfaces import ISchoolYearContainer
+from schooltool.skin.flourish.containers import TableContainerView
+from schooltool.skin.flourish.page import RefineLinksViewlet, Page
+from schooltool.skin.flourish.page import Content
+from schooltool.skin.flourish.page import ModalFormLinkViewlet
+from schooltool.skin.flourish.form import DialogForm, AddForm
 
 from schooltool.common import SchoolToolMessage as _
 
@@ -163,6 +178,22 @@ class ResourceContainerView(oldform.FormBase):
         formatter.cssClasses['table'] = 'data'
         return formatter()
 
+
+class FlourishResourceContainerView(TableContainerView):
+
+    @property
+    def done_link(self):
+        app = ISchoolToolApplication(None)
+        return absoluteURL(app, self.request) + '/manage'
+
+    def getColumnsAfter(self):
+        description = GetterColumn(
+            name='description',
+            title=_(u'Description'),
+            getter=lambda i, f: i.description or '')
+        return [description]
+
+
 class BaseTypeFilter(FilterWidget):
     """Base Type Filter"""
 
@@ -309,6 +340,44 @@ class ResourceContainerFilterWidget(PersonFilterWidget):
         return results
 
 
+class FlourishResourceContainerFilterWidget(ResourceContainerFilterWidget):
+
+    template = ViewPageTemplateFile('templates/f_resource_filter.pt')
+
+    def types(self):
+        options = [
+            {'id': 'equipment',
+             'title': _('Equipment')},
+            {'id': 'location',
+             'title': _('Location')},
+            {'id': 'resource',
+             'title': _('Resource')},
+            ]
+        return options
+
+    def filter(self, results):
+        if 'SEARCH_TITLE' in self.request:
+            searchstr = self.request['SEARCH_TITLE'].lower()
+            results = [item for item in results
+                       if searchstr in item.title.lower() or
+                       (item.description and searchstr in item.description.lower())]
+        if 'SEARCH_TYPE' in self.request:
+            type = self.request['SEARCH_TYPE']
+            if not type:
+                return results
+            results = [resource for resource in results
+                       if IResourceTypeInformation(resource).id == type]
+        return results
+
+
+class ResourceContainerLinks(RefineLinksViewlet):
+    """Resource container links viewlet."""
+
+
+class ResourceImportLinks(RefineLinksViewlet):
+    """Resource import links viewlet."""
+
+
 class ResourceDemographicsFieldsAbsoluteURLAdapter(BrowserView):
 
     adapts(IResourceDemographicsFields, IBrowserRequest)
@@ -327,6 +396,39 @@ class ResourceDemographicsView(DemographicsView):
     title = _('Resource Demographics Container')
 
 
+class FlourishResourceDemographicsView(FlourishDemographicsView):
+
+    container_class = 'container'
+
+    keys = [('resource', _('Res.')),
+            ('location', _('Locat.')),
+            ('equipment', _('Equip.'))]
+
+
+class FlourishReorderResourceDemographicsView(FlourishReorderDemographicsView):
+    pass
+
+
+@adapter(IResourceDemographicsFields)
+@implementer(IAddEditViewTitle)
+def getAddEditViewTitle(context):
+    return _('Resource attributes')
+
+
+@adapter(IResourceDemographicsFields)
+@implementer(ILimitKeysLabel)
+def getLimitKeysLabel(context):
+    return _('Limit to resource type(s)')
+
+
+@adapter(IResourceDemographicsFields)
+@implementer(ILimitKeysHint)
+def getLimitKeysHint(context):
+    return _(u"If you select one or more resource types below, this field "
+              "will only be displayed in forms and reports for "
+              "resources of the selected types.")
+
+
 ##########  Base class of all resource views (uses self.resource_type) #########
 class ResourceFieldGenerator(object):
 
@@ -391,6 +493,29 @@ class BaseResourceView(form.Form, ResourceFieldGenerator):
                 self.widgets[widget].mode = HIDDEN_MODE
 
 
+class FlourishBaseResourceView(Page, BaseResourceView):
+
+    def update(self):
+        BaseResourceView.update(self)
+
+    @property
+    def canModify(self):
+        return canAccess(self.context.__parent__, '__delitem__')
+
+    @property
+    def leaders_table(self):
+        return self.getTable(list(self.context.leaders))
+
+    def getTable(self, items):
+        persons = ISchoolToolApplication(None)['persons']
+        result = getMultiAdapter((persons, self.request), ITableFormatter)
+        result.setUp(table_formatter=table.StandaloneFullFormatter, items=items)
+        return result
+
+    def has_leaders(self):
+        return bool(list(self.context.leaders))
+
+
 class ResourceView(BaseResourceView):
     """A location info view."""
 
@@ -400,6 +525,12 @@ class ResourceView(BaseResourceView):
         return field.Fields(IResource)
 
 
+class FlourishResourceView(FlourishBaseResourceView, ResourceView):
+
+    def getBaseFields(self):
+        return field.Fields(IResource).select('title', 'description')
+
+
 class LocationView(BaseResourceView):
     """A location info view."""
 
@@ -409,6 +540,12 @@ class LocationView(BaseResourceView):
         return field.Fields(ILocation)
 
 
+class FlourishLocationView(FlourishBaseResourceView, LocationView):
+
+    def getBaseFields(self):
+        return field.Fields(ILocation).select('title', 'description')
+
+
 class EquipmentView(BaseResourceView):
     """A equipment info view."""
 
@@ -418,8 +555,22 @@ class EquipmentView(BaseResourceView):
         return field.Fields(IEquipment)
 
 
+class FlourishEquipmentView(FlourishBaseResourceView, EquipmentView):
+
+    def getBaseFields(self):
+        return field.Fields(IEquipment).select('title', 'description')
+
+
 ###############  Base classes of all resource add/edit views ################
-class BaseResourceAddView(form.AddForm, ResourceFieldGenerator):
+# XXX: move this to a generic form base class
+#      it's also duplicated in basicperson.browser.person.PersonForm
+class ErrorMessageBase(object):
+
+    formErrorsMessage = _('Please correct the marked fields below.')
+
+
+class BaseResourceAddView(ErrorMessageBase, form.AddForm,
+                          ResourceFieldGenerator):
 
     id = 'resource-form'
     template = ViewPageTemplateFile('templates/resource_form.pt')
@@ -466,7 +617,8 @@ class BaseResourceAddView(form.AddForm, ResourceFieldGenerator):
         return absoluteURL(self.context, self.request)
 
 
-class BaseResourceEditView(form.EditForm, ResourceFieldGenerator):
+class BaseResourceEditView(ErrorMessageBase, form.EditForm,
+                           ResourceFieldGenerator):
 
     id = 'resource-form'
     template = ViewPageTemplateFile('templates/resource_form.pt')
@@ -502,6 +654,31 @@ class BaseResourceEditView(form.EditForm, ResourceFieldGenerator):
         self.actions['cancel'].addClass('button-cancel')
 
 
+class BaseFlourishResourceAddForm(AddForm):
+
+    template = InheritTemplate(Page.template)
+    label = None
+
+    def createAndAdd(self, data):
+        resource = self._factory()
+        resource.title = data.get('title')
+        chooser = INameChooser(self.context)
+        resource.__name__ = chooser.chooseName('', resource)
+        form.applyChanges(self, resource, data)
+        notify(ObjectCreatedEvent(resource))
+        self.context[resource.__name__] = resource
+        self._resource = resource
+        return resource
+
+    @button.buttonAndHandler(_("Cancel"))
+    def handle_cancel_action(self, action):
+        url = absoluteURL(self.context, self.request)
+        self.request.response.redirect(url)
+
+    def nextURL(self):
+        return absoluteURL(self._resource, self.request)
+
+
 ###############  Resource add/edit views ################
 class BaseResourceForm(object):
 
@@ -512,7 +689,7 @@ class BaseResourceForm(object):
     demo_legend = _('Resource demographics')
 
     def getBaseFields(self):
-        return field.Fields(IResource).omit('type')
+        return field.Fields(IResource).omit('type', 'notes')
 
 
 class ResourceAddView(BaseResourceForm, BaseResourceAddView):
@@ -521,11 +698,25 @@ class ResourceAddView(BaseResourceForm, BaseResourceAddView):
     _factory = Resource
 
 
+class FlourishResourceAddView(BaseFlourishResourceAddForm, ResourceAddView):
+
+    demo_legend = _('Resource attributes')
+
+
 class ResourceEditView(BaseResourceForm, BaseResourceEditView):
 
     label = _('Edit resource')
 
 
+class FlourishResourceEditView(Page, ResourceEditView):
+
+    label = None
+    demo_legend = _('Resource attributes')
+
+    def update(self):
+        ResourceEditView.update(self)
+
+
 ###############  Location add/edit views ################
 class BaseLocationForm(object):
 
@@ -536,8 +727,7 @@ class BaseLocationForm(object):
     demo_legend = _('Location demographics')
 
     def getBaseFields(self):
-        fields = field.Fields(ILocation).select('title', 'type', 'description',
-            'capacity', 'notes')
+        fields = field.Fields(ILocation).select('title', 'description')
         return fields
 
 
@@ -547,11 +737,25 @@ class LocationAddView(BaseLocationForm, BaseResourceAddView):
     _factory = Location
 
 
+class FlourishLocationAddView(BaseFlourishResourceAddForm, LocationAddView):
+
+    demo_legend = _('Location attributes')
+
+
 class LocationEditView(BaseLocationForm, BaseResourceEditView):
 
     label = _('Edit location')
 
 
+class FlourishLocationEditView(Page, LocationEditView):
+
+    label = None
+    demo_legend = _('Location attributes')
+
+    def update(self):
+        LocationEditView.update(self)
+
+
 ###############  Equipment add/edit views ################
 class BaseEquipmentForm(object):
 
@@ -562,8 +766,7 @@ class BaseEquipmentForm(object):
     demo_legend = _('Equipment demographics')
 
     def getBaseFields(self):
-        fields = field.Fields(IEquipment).select('title', 'type', 'description',
-            'manufacturer', 'model', 'serialNumber', 'purchaseDate', 'notes')
+        fields = field.Fields(IEquipment).select('title', 'description')
         return fields
 
 
@@ -573,7 +776,121 @@ class EquipmentAddView(BaseEquipmentForm, BaseResourceAddView):
     _factory = Equipment
 
 
+class FlourishEquipmentAddView(BaseFlourishResourceAddForm, EquipmentAddView):
+
+    demo_legend = _('Equipment attributes')
+
+
 class EquipmentEditView(BaseEquipmentForm, BaseResourceEditView):
 
     label = _('Edit equipment')
 
+
+class FlourishEquipmentEditView(Page, EquipmentEditView):
+
+    label = None
+    demo_legend = _('Equipment attributes')
+
+    def update(self):
+        EquipmentEditView.update(self)
+
+
+class FlourishResourceContainerTableFormatter(SchoolToolTableFormatter):
+
+    # XXX: hack to customize the table class
+    def render(self):
+        formatter = self._table_formatter(
+            self.context, self.request, self._items,
+            columns=self._columns,
+            batch_start=self.batch.start, batch_size=self.batch.size,
+            sort_on=self._sort_on,
+            prefix=self.prefix)
+        formatter.cssClasses['table'] = 'resources-table'
+        return formatter()
+
+
+class ResourceLinks(RefineLinksViewlet):
+
+    @property
+    def title(self):
+        return self.context.title
+
+
+class ResourceActions(RefineLinksViewlet): pass
+
+
+class FlourishBookResourceView(BrowserView):
+
+    def __call__(self):
+        sessionWrapper = ISession(self.request)
+        session = sessionWrapper['schooltool.resource']
+        session['bookingSelection'] = [self.context.__name__]
+        url = absoluteURL(IBookingCalendar(self.context), self.request)
+        self.request.response.redirect(url)
+
+
+class FlourishResourceDeleteView(DialogForm, form.Form):
+
+    dialog_submit_actions = ('delete',)
+    dialog_close_actions = ('cancel',)
+    label = None
+
+    @button.buttonAndHandler(_('Delete'))
+    def handle_delete_action(self, action):
+        parent = self.context.__parent__
+        url = absoluteURL(parent, self.request)
+        url += '/delete.html?delete.%s&CONFIRM' % self.context.__name__
+        self.request.response.redirect(url)
+
+    @button.buttonAndHandler(_('Cancel'))
+    def handle_cancel_action(self, action):
+        pass
+
+    def updateActions(self):
+        super(FlourishResourceDeleteView, self).updateActions()
+        self.actions['delete'].addClass('button-ok')
+        self.actions['cancel'].addClass('button-cancel')
+
+
+class FlourishResourceDeleteLink(ModalFormLinkViewlet):
+
+    @property
+    def dialog_title(self):
+        title = _(u'Delete ${resource_title}',
+                  mapping={'resource_title': self.context.title})
+        return translate(title, context=self.request)
+
+
+class FlourishManageResourcesOverview(Content):
+
+    body_template = ViewPageTemplateFile(
+        'templates/f_manage_resources_overview.pt')
+
+    @property
+    def schoolyear(self):
+        schoolyears = ISchoolYearContainer(self.context)
+        result = schoolyears.getActiveSchoolYear()
+        if 'schoolyear_id' in self.request:
+            schoolyear_id = self.request['schoolyear_id']
+            result = schoolyears.get(schoolyear_id, result)
+        return result
+
+    @property
+    def has_schoolyear(self):
+        return self.schoolyear is not None
+
+    @property
+    def resources(self):
+        return self.context['resources']
+
+    @Lazy
+    def resource_types(self):
+        types = defaultdict(lambda:dict(amount=0, title=None, id=None))
+        for resource in self.resources.values():
+            info = IResourceTypeInformation(resource)
+            if types[info.id]['title'] is None:
+                types[info.id]['title'] = info.title
+            if types[info.id]['id'] is None:
+                types[info.id]['id'] = info.id
+            types[info.id]['amount'] += 1
+        return types
